/*
 * ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Apache Software Foundation (http://www.apache.org/)."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 * "Apache JMeter" must not be used to endorse or promote products
 * derived from this software without prior written permission. For
 * written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 * "Apache JMeter", nor may "Apache" appear in their name, without
 * prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package org.apache.jmeter.visualizers;

import java.awt.*;
import java.util.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

import org.apache.jmeter.samplers.SampleResult;
import org.apache.jmeter.gui.*;
import org.apache.jmeter.util.JMeterUtils;

/**
 *  This class implements a statistical analyser that calculates both the
 *  average and the standard deviation of the sampling process and outputs them
 *  as autoscaling plots.
 *
 *@author     <a href="mailto:stefano@apache.org">Stefano Mazzocchi</a>
 *@created    February 8, 2001
 *@version    $Revision: 1.17 $ $Date: 2002/02/23 01:21:07 $
 */
public class GraphVisualizer extends JPanel
		 implements ImageVisualizer, ItemListener,ModelSupported,GraphListener,
		GenericGraphListener
{
	private GraphModel model;

	private NamePanel namePanel;

	private Graph graph;
	private JCheckBox data;
	private JCheckBox average;
	private JCheckBox deviation;
	private JTextField dataField;
	private JTextField averageField;
	private JTextField deviationField;

	/**
	 *  Constructor for the GraphVisualizer object
	 */
	public GraphVisualizer()
	{
		super();
	}

	public void setModel(Object model)
	{
		this.model = (GraphModel)model;
		this.model.addModelListener(this);
		init();
	}

	public void updateGui()
	{
		namePanel.updateGui();
		graph.updateGui();
		dataField.setText(Long.toString(model.getCurrentData()));
		averageField.setText(Long.toString(model.getCurrentAverage()));
		deviationField.setText(Long.toString(model.getCurrentDeviation()));
	}

	public void updateGui(Sample s)
	{
		dataField.setText(Long.toString(s.data));
		averageField.setText(Long.toString(s.average));
		deviationField.setText(Long.toString(s.deviation));
	}

	private void init()
	{
		this.setLayout(new BorderLayout());

		// MAIN PANEL
		JPanel mainPanel = new JPanel();
		Border margin = new EmptyBorder(10, 10, 5, 10);
		mainPanel.setBorder(margin);
		mainPanel.setLayout(new VerticalLayout(5, VerticalLayout.LEFT));

		// TITLE
		JLabel panelTitleLabel = new JLabel(JMeterUtils.getResString("graph_results_title"));
		Font curFont = panelTitleLabel.getFont();
		int curFontSize = curFont.getSize();
		curFontSize += 4;
		panelTitleLabel.setFont(new Font(curFont.getFontName(), curFont.getStyle(), curFontSize));
		mainPanel.add(panelTitleLabel);

		// NAME
		namePanel = new NamePanel(model);
		mainPanel.add(namePanel);

		data = new JCheckBox(JMeterUtils.getResString("graph_results_data"));
		data.setSelected(true);
		data.addItemListener(this);
		data.setForeground(Color.black);
		average = new JCheckBox(JMeterUtils.getResString("graph_results_average"));
		average.setSelected(true);
		average.addItemListener(this);
		average.setForeground(Color.blue);
		deviation = new JCheckBox(JMeterUtils.getResString("graph_results_deviation"));
		deviation.setSelected(true);
		deviation.addItemListener(this);
		deviation.setForeground(Color.red);

		JPanel checkPanel = new JPanel();
		checkPanel.setLayout(new GridLayout(0, 1));
		checkPanel.add(data);
		checkPanel.add(average);
		checkPanel.add(deviation);

		graph = new Graph(model);

		JScrollPane graphScrollPanel = new JScrollPane(graph,JScrollPane.VERTICAL_SCROLLBAR_NEVER,
				JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		graphScrollPanel.setViewportBorder(BorderFactory.createEmptyBorder(2, 2, 2, 2));
		//graphScrollPanel.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		//graphScrollPanel.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_NEVER);

		dataField = new JTextField(5);
		dataField.setEditable(false);
		dataField.setForeground(Color.black);
		dataField.setBackground(getBackground());
		averageField = new JTextField(5);
		averageField.setEditable(false);
		averageField.setForeground(Color.blue);
		averageField.setBackground(getBackground());
		deviationField = new JTextField(5);
		deviationField.setEditable(false);
		deviationField.setForeground(Color.red);
		deviationField.setBackground(getBackground());

		JPanel showPanel = new JPanel();
		showPanel.setLayout(new GridLayout(0, 1));
		showPanel.setBorder(BorderFactory.createEmptyBorder(0, 5, 0, 0));
		showPanel.add(dataField);
		showPanel.add(averageField);
		showPanel.add(deviationField);

		this.add(mainPanel,BorderLayout.NORTH);
		this.add(checkPanel, BorderLayout.WEST);
		this.add(graphScrollPanel, BorderLayout.CENTER);
		this.add(showPanel, BorderLayout.EAST);
	}

	/**
	 *  Gets the Image attribute of the GraphVisualizer object
	 *
	 *@return    The Image value
	 */
	public Image getImage()
	{
		Image result = graph.createImage(graph.getWidth(), graph.getHeight());
		graph.paintComponent(result.getGraphics());

		return result;
	}

	/**
	 *  Description of the Method
	 *
	 *@param  e  Description of Parameter
	 */
	public void itemStateChanged(ItemEvent e)
	{
		if (e.getItem() == data)
		{
			this.graph.enableData(e.getStateChange() == ItemEvent.SELECTED);
		}
		else if (e.getItem() == average)
		{
			this.graph.enableAverage(e.getStateChange() == ItemEvent.SELECTED);
		}
		else if (e.getItem() == deviation)
		{
			this.graph.enableDeviation(e.getStateChange() == ItemEvent.SELECTED);
		}
		this.graph.repaint();
	}


	/**
	 *  Description of the Method
	 */
	public synchronized void clear()
	{
		//this.graph.clear();

		dataField.setText("0000");
		averageField.setText("0000");
		deviationField.setText("0000");

		repaint();
	}

	/**
	 *  Description of the Method
	 *
	 *@return    Description of the Returned Value
	 */
	public String toString()
	{
		return "Show the samples analysys as dot plots";
	}




}
